#!/usr/bin/python

import numpy as np
import scipy as sp
import scipy.fftpack as fp
import sys
import csv

from sklearn import cross_validation as cv
from sklearn import preprocessing as pp
from sklearn.ensemble import RandomForestClassifier
from sklearn.grid_search import GridSearchCV
from sklearn.cross_validation import StratifiedKFold

def gridSearch(selector, param_grid, train, target):
    cvk = StratifiedKFold(target, n_folds=3)
    grid = GridSearchCV(selector, param_grid, n_jobs=-1, cv = cvk)
    grid.fit(train, target)
    return grid


np.seterr(divide='ignore', invalid='ignore')

DATASETS = [
    "ChlorineConcentration",
    "wafer",
    "MedicalImages",
    "FaceAll",
    "OSULeaf",
    "Adiac",
    "SwedishLeaf",
    "yoga",
    "fish",
    "Lighting7",
    "Lighting2",
    "Trace",
    "synthetic_control",
    "FacesUCR",
    "CinC_ECG_torso",
    "MALLAT",
    "Symbols",
    "Coffee",
    "ECG200",
    "FaceFour",
    "OliveOil",
    "Gun_Point",
    "Beef",
    "DiatomSizeReduction",
    "CBF",
    "ECGFiveDays",
    "TwoLeadECG",
    "SonyAIBORobot SurfaceII",
    "MoteStrain",
    "ItalyPowerDemand",
    "SonyAIBORobot Surface",
    "Two_Patterns",
    "StarLightCurves",
    "Haptics",
    "InlineSkate",
    "50words",
    "Cricket_Y",
    "Cricket_X",
    "Cricket_Z",
    "WordsSynonyms",
    "uWaveGestureLibrary_Z",
    "uWaveGestureLibrary_Y",
    "uWaveGestureLibrary_X"
  ]

DATA   = "../../datasets/classification/"


norm = pp.Normalizer(norm="l2")

for d in DATASETS:
    print d, 

    train = np.array(np.genfromtxt(open(DATA + d + '/' + d + '_TRAIN','rb'), delimiter=None))
    test = np.array(np.genfromtxt(open(DATA  + d + '/' + d + '_TEST','rb'), delimiter=None))
    
    labels_train = train[:,0]
    labels_test = test[:,0]
    
    train = train[:,1:]
    test = test[:,1:]
    
    test = norm.transform(test)
    train = norm.transform(train)
    
    n_estimators_range = np.arange(10, 50, step=5)
    
    param_grid = {
                  "n_estimators": n_estimators_range.tolist(),
                  "random_state": [1]
                  }
    
    grid = gridSearch(RandomForestClassifier(), param_grid, train, labels_train)
    classifier = grid.best_estimator_                
    best_params = grid.best_params_
    
    classifier.fit(train, labels_train)
    pre_labels = classifier.predict(test)
    
    train_accuracy = np.mean(labels_test == pre_labels)
    print('\tscore:\t%0.5f' % (train_accuracy))
            
        
        